@@ -2,6 +2,7 @@ use std::borrow::Borrow;
 use std::fmt::Debug;
 use std::marker::PhantomData;
 use std::ops::Range;
+use std::sync::Arc;
 
 use super::dual_coord::DualCoordChartContext;
 use super::mesh::MeshStyle;
@@ -12,10 +13,13 @@ use crate::coord::{
 };
 use crate::drawing::backend::{BackendCoord, DrawingBackend};
 use crate::drawing::{DrawingArea, DrawingAreaErrorKind};
-use crate::element::{Drawable, DynElement, IntoDynElement, Path, PointCollection};
+use crate::element::{Drawable, DynElement, IntoDynElement, PathElement, PointCollection};
+use crate::style::text_anchor::{HPos, Pos, VPos};
 use crate::style::{AsRelative, FontTransform, ShapeStyle, SizeDesc, TextStyle};
 
 /// The annotations (such as the label of the series, the legend element, etc)
+/// When a series is drawn onto a drawing area, an series annotation object
+/// is created and a mutable reference is returned.
 #[allow(clippy::type_complexity)]
 pub struct SeriesAnno<'a, DB: DrawingBackend> {
     label: Option<String>,
@@ -25,7 +29,7 @@ pub struct SeriesAnno<'a, DB: DrawingBackend> {
 
 impl<'a, DB: DrawingBackend> SeriesAnno<'a, DB> {
     pub(crate) fn get_label(&self) -> &str {
-        self.label.as_ref().map(|x| x.as_str()).unwrap_or("")
+        self.label.as_deref().map(|x| x).unwrap_or("")
     }
 
     pub(crate) fn get_draw_func(
@@ -43,7 +47,7 @@ impl<'a, DB: DrawingBackend> SeriesAnno<'a, DB> {
     }
 
     /// Set the series label
-    /// `label`: The string would be use as label for current series
+    /// - `label`: The string would be use as label for current series
     pub fn label<L: Into<String>>(&mut self, label: L) -> &mut Self {
         self.label = Some(label.into());
         self
@@ -70,6 +74,102 @@ pub struct ChartContext<'a, DB: DrawingBackend, CT: CoordTranslate> {
     pub(super) y_label_area: [Option<DrawingArea<DB, Shift>>; 2],
     pub(super) drawing_area: DrawingArea<DB, CT>,
     pub(super) series_anno: Vec<SeriesAnno<'a, DB>>,
+    pub(super) drawing_area_pos: (i32, i32),
+}
+
+/// A chart context state - This is the data that is needed to reconstruct the chart context
+/// without actually drawing the chart. This is useful when we want to do realtime rendering and
+/// want to incrementally update the chart.
+///
+/// For each frame, instead of updating the entire backend, we are able to keep the keep the figure
+/// component like axis, labels untouched and make updates only in the plotting drawing area.
+pub struct ChartState<CT: CoordTranslate> {
+    drawing_area_pos: (i32, i32),
+    drawing_area_size: (u32, u32),
+    coord: CT,
+}
+
+impl<'a, CT: CoordTranslate + Clone> Clone for ChartState<CT> {
+    fn clone(&self) -> Self {
+        Self {
+            drawing_area_size: self.drawing_area_size,
+            drawing_area_pos: self.drawing_area_pos,
+            coord: self.coord.clone(),
+        }
+    }
+}
+
+impl<'a, DB: DrawingBackend, CT: CoordTranslate> From<ChartContext<'a, DB, CT>> for ChartState<CT> {
+    fn from(chart: ChartContext<'a, DB, CT>) -> ChartState<CT> {
+        ChartState {
+            drawing_area_pos: chart.drawing_area_pos,
+            drawing_area_size: chart.drawing_area.dim_in_pixel(),
+            coord: chart.drawing_area.into_coord_spec(),
+        }
+    }
+}
+
+impl<'a, DB: DrawingBackend, CT: CoordTranslate> ChartContext<'a, DB, CT> {
+    /// Convert a chart context into a chart state, by doing so, the chart context is consumed and
+    /// a saved chart state is created for later use.
+    pub fn into_chart_state(self) -> ChartState<CT> {
+        self.into()
+    }
+
+    /// Convert the chart context into a sharable chart state.
+    /// Normally a chart state can not be clone, since the coordinate spec may not be able to be
+    /// cloned. In this case, we can use an `Arc` get the coordinate wrapped thus the state can be
+    /// cloned and shared by multiple chart context
+    pub fn into_shared_chart_state(self) -> ChartState<Arc<CT>> {
+        ChartState {
+            drawing_area_pos: self.drawing_area_pos,
+            drawing_area_size: self.drawing_area.dim_in_pixel(),
+            coord: Arc::new(self.drawing_area.into_coord_spec()),
+        }
+    }
+}
+
+impl<'a, 'b, DB, CT> From<&ChartContext<'a, DB, CT>> for ChartState<CT>
+where
+    DB: DrawingBackend,
+    CT: CoordTranslate + Clone,
+{
+    fn from(chart: &ChartContext<'a, DB, CT>) -> ChartState<CT> {
+        ChartState {
+            drawing_area_pos: chart.drawing_area_pos,
+            drawing_area_size: chart.drawing_area.dim_in_pixel(),
+            coord: chart.drawing_area.as_coord_spec().clone(),
+        }
+    }
+}
+
+impl<'a, DB: DrawingBackend, CT: CoordTranslate + Clone> ChartContext<'a, DB, CT> {
+    /// Make the chart context, do not consume the chart context and clone the coordinate spec
+    pub fn to_chart_state(&self) -> ChartState<CT> {
+        self.into()
+    }
+}
+
+impl<CT: CoordTranslate> ChartState<CT> {
+    /// Restore the chart context on the given drawing area
+    ///
+    /// - `area`: The given drawing area where we want to restore the chart context
+    /// - **returns** The newly created chart context
+    pub fn restore<'a, DB: DrawingBackend>(
+        self,
+        area: &DrawingArea<DB, Shift>,
+    ) -> ChartContext<'a, DB, CT> {
+        let area = area
+            .clone()
+            .shrink(self.drawing_area_pos, self.drawing_area_size);
+        ChartContext {
+            x_label_area: [None, None],
+            y_label_area: [None, None],
+            drawing_area: area.apply_coord_spec(self.coord),
+            series_anno: vec![],
+            drawing_area_pos: self.drawing_area_pos,
+        }
+    }
 }
 
 impl<
@@ -82,23 +182,21 @@ impl<
     > ChartContext<'a, DB, RangedCoord<X, Y>>
 {
     fn is_overlapping_drawing_area(&self, area: Option<&DrawingArea<DB, Shift>>) -> bool {
-        let area = if area.is_none() {
-            return false;
+        if let Some(area) = area {
+            let (x0, y0) = area.get_base_pixel();
+            let (w, h) = area.dim_in_pixel();
+            let (x1, y1) = (x0 + w as i32, y0 + h as i32);
+            let (dx0, dy0) = self.drawing_area.get_base_pixel();
+            let (w, h) = self.drawing_area.dim_in_pixel();
+            let (dx1, dy1) = (dx0 + w as i32, dy0 + h as i32);
+
+            let (ox0, ox1) = (x0.max(dx0), x1.min(dx1));
+            let (oy0, oy1) = (y0.max(dy0), y1.min(dy1));
+
+            ox1 > ox0 && oy1 > oy0
         } else {
-            area.unwrap()
-        };
-
-        let (x0, y0) = area.get_base_pixel();
-        let (w, h) = area.dim_in_pixel();
-        let (x1, y1) = (x0 + w as i32, y0 + h as i32);
-        let (dx0, dy0) = self.drawing_area.get_base_pixel();
-        let (w, h) = self.drawing_area.dim_in_pixel();
-        let (dx1, dy1) = (dx0 + w as i32, dy0 + h as i32);
-
-        let (ox0, ox1) = (x0.max(dx0), x1.min(dx1));
-        let (oy0, oy1) = (y0.max(dy0), y1.min(dy1));
-
-        ox1 > ox0 && oy1 > oy0
+            false
+        }
     }
 
     /// Initialize a mesh configuration object and mesh drawing can be finalized by calling
@@ -131,7 +229,8 @@ impl<
             n_y_labels: 10,
             line_style_1: None,
             line_style_2: None,
-            label_style: None,
+            x_label_style: None,
+            y_label_style: None,
             format_x: &|x| format!("{:?}", x),
             format_y: &|y| format!("{:?}", y),
             target: Some(self),
@@ -157,6 +256,12 @@ impl<'a, DB: DrawingBackend + 'a, CT: CoordTranslate> ChartContext<'a, DB, CT> {
     }
 }
 
+impl<'a, DB: DrawingBackend, CT: CoordTranslate> ChartContext<'a, DB, CT> {
+    pub fn as_coord_spec(&self) -> &CT {
+        self.drawing_area.as_coord_spec()
+    }
+}
+
 impl<'a, DB: DrawingBackend, CT: ReverseCoordTranslate> ChartContext<'a, DB, CT> {
     /// Convert the chart context into an closure that can be used for coordinate translation
     pub fn into_coord_trans(self) -> impl Fn(BackendCoord) -> Option<CT::From> {
@@ -165,6 +270,45 @@ impl<'a, DB: DrawingBackend, CT: ReverseCoordTranslate> ChartContext<'a, DB, CT>
     }
 }
 
+impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, Arc<RangedCoord<X, Y>>> {
+    pub(super) fn draw_series_impl<E, R, S>(
+        &mut self,
+        series: S,
+    ) -> Result<(), DrawingAreaErrorKind<DB::ErrorType>>
+    where
+        for<'b> &'b E: PointCollection<'b, (X::ValueType, Y::ValueType)>,
+        E: Drawable<DB>,
+        R: Borrow<E>,
+        S: IntoIterator<Item = R>,
+    {
+        for element in series {
+            self.drawing_area.draw(element.borrow())?;
+        }
+        Ok(())
+    }
+
+    pub(super) fn alloc_series_anno(&mut self) -> &mut SeriesAnno<'a, DB> {
+        let idx = self.series_anno.len();
+        self.series_anno.push(SeriesAnno::new());
+        &mut self.series_anno[idx]
+    }
+
+    /// Draw a data series. A data series in Plotters is abstracted as an iterator of elements
+    pub fn draw_series<E, R, S>(
+        &mut self,
+        series: S,
+    ) -> Result<&mut SeriesAnno<'a, DB>, DrawingAreaErrorKind<DB::ErrorType>>
+    where
+        for<'b> &'b E: PointCollection<'b, (X::ValueType, Y::ValueType)>,
+        E: Drawable<DB>,
+        R: Borrow<E>,
+        S: IntoIterator<Item = R>,
+    {
+        self.draw_series_impl(series)?;
+        Ok(self.alloc_series_anno())
+    }
+}
+
 impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCoord<X, Y>> {
     /// Get the range of X axis
     pub fn x_range(&self) -> Range<X::ValueType> {
@@ -291,23 +435,23 @@ impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCo
         }
 
         if let Some(axis_style) = axis_style {
-            let mut x0 = if orientation.0 > 0 { 0 } else { tw as i32 };
-            let mut y0 = if orientation.1 > 0 { 0 } else { th as i32 };
-            let mut x1 = if orientation.0 >= 0 { 0 } else { tw as i32 };
-            let mut y1 = if orientation.1 >= 0 { 0 } else { th as i32 };
+            let mut x0 = if orientation.0 > 0 { 0 } else { tw as i32 - 1 };
+            let mut y0 = if orientation.1 > 0 { 0 } else { th as i32 - 1 };
+            let mut x1 = if orientation.0 >= 0 { 0 } else { tw as i32 - 1 };
+            let mut y1 = if orientation.1 >= 0 { 0 } else { th as i32 - 1 };
 
             if inward_labels {
                 if orientation.0 == 0 {
                     if y0 == 0 {
-                        y0 = th as i32;
-                        y1 = th as i32;
+                        y0 = th as i32 - 1;
+                        y1 = th as i32 - 1;
                     } else {
                         y0 = 0;
                         y1 = 0;
                     }
                 } else if x0 == 0 {
-                    x0 = tw as i32;
-                    x1 = tw as i32;
+                    x0 = tw as i32 - 1;
+                    x1 = tw as i32 - 1;
                 } else {
                     x0 = 0;
                     x1 = 0;
@@ -322,29 +466,15 @@ impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCo
                 y1 = axis_range.end;
             }
 
-            area.draw(&Path::new(vec![(x0, y0), (x1, y1)], axis_style.clone()))?;
+            area.draw(&PathElement::new(
+                vec![(x0, y0), (x1, y1)],
+                axis_style.clone(),
+            ))?;
         }
 
         Ok(axis_range)
     }
 
-    fn estimate_right_aligned_label_offset(
-        &self,
-        label_style: &TextStyle,
-        labels: &[(i32, String)],
-    ) -> i32 {
-        labels
-            .iter()
-            .map(|(_, t)| {
-                self.drawing_area
-                    .estimate_text_size(t, &label_style.font)
-                    .unwrap_or((0, 0))
-                    .0
-            })
-            .max()
-            .unwrap_or(0) as i32
-    }
-
     // TODO: consider make this function less complicated
     #[allow(clippy::too_many_arguments)]
     #[allow(clippy::cognitive_complexity)]
@@ -369,28 +499,43 @@ impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCo
         let (tw, th) = area.dim_in_pixel();
 
         /* This is the minimal distance from the axis to the box of the labels */
-        let label_dist =
-            if (tick_size > 0 && orientation.1 > 0) || (tick_size < 0 && orientation.1 < 0) {
-                0
-            } else {
-                tick_size.abs() * 2
-            };
+        let label_dist = tick_size.abs() * 2;
 
         /* Draw the axis and get the axis range so that we can do further label
          * and tick mark drawing */
         let axis_range = self.draw_axis(area, axis_style, orientation, tick_size < 0)?;
 
-        /* If the label area is on the right hand side, we should enable the right aligned
-         * layout, thus in this case we need to estimate the right most position when all
-         * the labels are right aligned */
-        let right_alignment = if orientation.0 > 0 && orientation.1 == 0 {
-            self.estimate_right_aligned_label_offset(label_style, labels)
-        } else {
-            0
-        };
+        /* To make the right label area looks nice, it's a little bit tricky, since for a that is
+         * very long, we actually prefer left alignment instead of right alignment.
+         * Otherwise, the right alignment looks better. So we estimate the max and min label width
+         * So that we are able decide if we should apply right alignment for the text. */
+        let label_width: Vec<_> = labels
+            .iter()
+            .map(|(_, text)| {
+                if orientation.0 > 0 && orientation.1 == 0 && tick_size >= 0 {
+                    let ((x0, _), (x1, _)) = label_style
+                        .font
+                        .layout_box(text)
+                        .unwrap_or(((0, 0), (0, 0)));
+                    x1 - x0
+                } else {
+                    // Don't ever do the layout estimationfor the drawing area that is either not
+                    // the right one or the tick mark is inward.
+                    0
+                }
+            })
+            .collect();
+
+        let min_width = *label_width.iter().min().unwrap_or(&1).max(&1);
+        let max_width = *label_width
+            .iter()
+            .filter(|&&x| x < min_width * 2)
+            .max()
+            .unwrap_or(&min_width);
+        let right_align_width = (min_width * 2).min(max_width);
 
         /* Then we need to draw the tick mark and the label */
-        for (p, t) in labels {
+        for ((p, t), w) in labels.iter().zip(label_width.into_iter()) {
             /* Make sure we are actually in the visible range */
             let rp = if orientation.0 == 0 { *p - x0 } else { *p - y0 };
 
@@ -400,84 +545,92 @@ impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCo
                 continue;
             }
 
-            /* Then we need to estimate the text if rendered */
-            let (w, h) = self
-                .drawing_area
-                .estimate_text_size(&t, &label_style.font)
-                .unwrap_or((0, 0));
-
-            let (cx, cy) = if tick_size >= 0 {
+            let (cx, cy, h_pos, v_pos) = if tick_size >= 0 {
                 match orientation {
                     // Right
                     (dx, dy) if dx > 0 && dy == 0 => {
-                        (right_alignment + label_dist - w as i32, *p - y0)
+                        if w >= right_align_width {
+                            (label_dist, *p - y0, HPos::Left, VPos::Center)
+                        } else {
+                            (
+                                label_dist + right_align_width,
+                                *p - y0,
+                                HPos::Right,
+                                VPos::Center,
+                            )
+                        }
                     }
                     // Left
-                    (dx, dy) if dx < 0 && dy == 0 => (tw as i32 - label_dist - w as i32, *p - y0),
+                    (dx, dy) if dx < 0 && dy == 0 => {
+                        (tw as i32 - label_dist, *p - y0, HPos::Right, VPos::Center)
+                    }
                     // Bottom
-                    (dx, dy) if dx == 0 && dy > 0 => (*p - x0, label_dist + h as i32),
+                    (dx, dy) if dx == 0 && dy > 0 => (*p - x0, label_dist, HPos::Center, VPos::Top),
                     // Top
-                    (dx, dy) if dx == 0 && dy < 0 => (*p - x0, th as i32 - label_dist - h as i32),
+                    (dx, dy) if dx == 0 && dy < 0 => {
+                        (*p - x0, th as i32 - label_dist, HPos::Center, VPos::Bottom)
+                    }
                     _ => panic!("Bug: Invalid orientation specification"),
                 }
             } else {
                 match orientation {
                     // Right
                     (dx, dy) if dx > 0 && dy == 0 => {
-                        (tw as i32 - right_alignment - label_dist, *p - y0)
+                        (tw as i32 - label_dist, *p - y0, HPos::Right, VPos::Center)
                     }
                     // Left
-                    (dx, dy) if dx < 0 && dy == 0 => (label_dist, *p - y0),
+                    (dx, dy) if dx < 0 && dy == 0 => {
+                        (label_dist, *p - y0, HPos::Left, VPos::Center)
+                    }
                     // Bottom
-                    (dx, dy) if dx == 0 && dy > 0 => (*p - x0, th as i32 - label_dist - h as i32),
+                    (dx, dy) if dx == 0 && dy > 0 => {
+                        (*p - x0, th as i32 - label_dist, HPos::Center, VPos::Bottom)
+                    }
                     // Top
-                    (dx, dy) if dx == 0 && dy < 0 => (*p - x0, label_dist + h as i32),
+                    (dx, dy) if dx == 0 && dy < 0 => (*p - x0, label_dist, HPos::Center, VPos::Top),
                     _ => panic!("Bug: Invalid orientation specification"),
                 }
             };
 
-            let should_draw = if orientation.0 == 0 {
-                cx >= 0 && cx + label_offset + w as i32 / 2 <= tw as i32
+            let (text_x, text_y) = if orientation.0 == 0 {
+                (cx + label_offset, cy)
             } else {
-                cy >= 0 && cy + label_offset + h as i32 / 2 <= th as i32
+                (cx, cy + label_offset)
             };
 
-            if should_draw {
-                let (text_x, text_y) = if orientation.0 == 0 {
-                    (cx - w as i32 / 2 + label_offset, cy)
-                } else {
-                    (cx, cy - h as i32 / 2 + label_offset)
-                };
-
-                area.draw_text(&t, label_style, (text_x, text_y))?;
+            let label_style = &label_style.pos(Pos::new(h_pos, v_pos));
+            area.draw_text(&t, label_style, (text_x, text_y))?;
 
+            if tick_size != 0 {
                 if let Some(style) = axis_style {
+                    let xmax = tw as i32 - 1;
+                    let ymax = th as i32 - 1;
                     let (kx0, ky0, kx1, ky1) = if tick_size > 0 {
                         match orientation {
                             (dx, dy) if dx > 0 && dy == 0 => (0, *p - y0, tick_size, *p - y0),
                             (dx, dy) if dx < 0 && dy == 0 => {
-                                (tw as i32 - tick_size, *p - y0, tw as i32, *p - y0)
+                                (xmax - tick_size, *p - y0, xmax, *p - y0)
                             }
                             (dx, dy) if dx == 0 && dy > 0 => (*p - x0, 0, *p - x0, tick_size),
                             (dx, dy) if dx == 0 && dy < 0 => {
-                                (*p - x0, th as i32 - tick_size, *p - x0, th as i32)
+                                (*p - x0, ymax - tick_size, *p - x0, ymax)
                             }
                             _ => panic!("Bug: Invalid orientation specification"),
                         }
                     } else {
                         match orientation {
                             (dx, dy) if dx > 0 && dy == 0 => {
-                                (tw as i32, *p - y0, tw as i32 + tick_size, *p - y0)
+                                (xmax, *p - y0, xmax + tick_size, *p - y0)
                             }
                             (dx, dy) if dx < 0 && dy == 0 => (0, *p - y0, -tick_size, *p - y0),
                             (dx, dy) if dx == 0 && dy > 0 => {
-                                (*p - x0, th as i32, *p - x0, th as i32 + tick_size)
+                                (*p - x0, ymax, *p - x0, ymax + tick_size)
                             }
                             (dx, dy) if dx == 0 && dy < 0 => (*p - x0, 0, *p - x0, -tick_size),
                             _ => panic!("Bug: Invalid orientation specification"),
                         }
                     };
-                    let line = Path::new(vec![(kx0, ky0), (kx1, ky1)], style.clone());
+                    let line = PathElement::new(vec![(kx0, ky0), (kx1, ky1)], style.clone());
                     area.draw(&line)?;
                 }
             }
@@ -492,19 +645,19 @@ impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCo
                 style.transform(FontTransform::Rotate90)
             };
 
-            let (w, h) = self
-                .drawing_area
-                .estimate_text_size(text, &actual_style.font)
-                .unwrap_or((0, 0));
-
-            let (x0, y0) = match orientation {
-                (dx, dy) if dx > 0 && dy == 0 => (tw - w, (th - h) / 2),
-                (dx, dy) if dx < 0 && dy == 0 => (0, (th - h) / 2),
-                (dx, dy) if dx == 0 && dy > 0 => ((tw - w) / 2, th - h),
-                (dx, dy) if dx == 0 && dy < 0 => ((tw - w) / 2, 0),
+            let (x0, y0, h_pos, v_pos) = match orientation {
+                // Right
+                (dx, dy) if dx > 0 && dy == 0 => (tw, th / 2, HPos::Center, VPos::Top),
+                // Left
+                (dx, dy) if dx < 0 && dy == 0 => (0, th / 2, HPos::Center, VPos::Top),
+                // Bottom
+                (dx, dy) if dx == 0 && dy > 0 => (tw / 2, th, HPos::Center, VPos::Bottom),
+                // Top
+                (dx, dy) if dx == 0 && dy < 0 => (tw / 2, 0, HPos::Center, VPos::Top),
                 _ => panic!("Bug: Invalid orientation specification"),
             };
 
+            let actual_style = &actual_style.pos(Pos::new(h_pos, v_pos));
             area.draw_text(&text, &actual_style, (x0 as i32, y0 as i32))?;
         }
 
@@ -516,7 +669,8 @@ impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCo
         &mut self,
         (r, c): (usize, usize),
         mesh_line_style: &ShapeStyle,
-        label_style: &TextStyle,
+        x_label_style: &TextStyle,
+        y_label_style: &TextStyle,
         fmt_label: FmtLabel,
         x_mesh: bool,
         y_mesh: bool,
@@ -542,7 +696,7 @@ impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCo
                 self.x_label_area[idx].as_ref(),
                 if x_axis { Some(axis_style) } else { None },
                 &x_labels[..],
-                label_style,
+                x_label_style,
                 x_label_offset,
                 (0, -1 + idx as i16 * 2),
                 x_desc.as_ref().map(|desc| (&desc[..], axis_desc_style)),
@@ -553,7 +707,7 @@ impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCo
                 self.y_label_area[idx].as_ref(),
                 if y_axis { Some(axis_style) } else { None },
                 &y_labels[..],
-                label_style,
+                y_label_style,
                 y_label_offset,
                 (-1 + idx as i16 * 2, 0),
                 y_desc.as_ref().map(|desc| (&desc[..], axis_desc_style)),
@@ -564,7 +718,11 @@ impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCo
         Ok(())
     }
 
-    /// Attach a secondary coord to the chart
+    /// Convert this chart context into a dual axis chart context
+    ///
+    /// - `x_coord`: The coordinate spec for the X axis
+    /// - `y_coord`: The coordinate spec for the Y axis
+    /// - **returns** The newly created dual spec chart context
     #[allow(clippy::type_complexity)]
     pub fn set_secondary_coord<SX: AsRangedCoord, SY: AsRangedCoord>(
         self,
